@@ -14,8 +14,82 @@ import sys
 import tempfile
 import os
 import codecs
+import datetime
+from random import randint # make it possible to do testdata.randint so 2 imports aren't needed
+from collections import deque
+import types
+
+__version__ = '0.5.3'
+
+def create_file_structure(path_str, tmpdir=u""):
+    """
+    create a whole file structure with a string, one file/directory per line
+
+    NOTE -- the directories always have to end with /
+
+    path_str -- string -- a multiline python string
+    tmpdir -- string -- the root
+    """
+    path_lines = []
+    if isinstance(path_str, types.StringTypes):
+        path_lines = filter(None, path_str.split(os.linesep))
+    else:
+        path_lines = path_str
+
+    dirs = set() # directories will always end with /
+    files = set()
+    depth_queue = deque()
+    depth_queue.appendleft(0)
+    path_prefix = deque()
+    path_prefix.appendleft([])
+
+    for path in path_lines:
+        # get the size of the whitespace:
+        m = re.match("^\s*", path)
+        current_depth = len(m.group(0))
+        if current_depth > depth_queue[0]:
+            depth_queue.appendleft(current_depth)
+        elif current_depth < depth_queue[0]:
+            depth = depth_queue[0]
+            while current_depth < depth:
+                depth = depth_queue.popleft()
+                path_prefix.popleft()
+
+            if len(path_prefix) == 0:
+                path_prefix.appendleft([])
+            if len(depth_queue) == 0:
+                depth_queue.appendleft(0)
+
+        p = path.strip()
+        is_dir = p.endswith("/") or p.endswith(os.sep)
+        p = _normpath(p)
+        if p:
+            full_path = filter(None, list(path_prefix[0]) + p.split(os.sep))
+            if is_dir:
+                path_prefix.appendleft(full_path)
+                dirs.add(os.sep.join(full_path))
+            else:
+                file_path = os.sep.join(full_path)
+                dir_path = os.path.dirname(file_path)
+                files.add(file_path)
+                dirs.add(dir_path)
+
+                full_dir_path = filter(None, dir_path.split(os.sep))
+                if path_prefix[0] != full_dir_path:
+                    path_prefix.appendleft(full_dir_path)
+
+    if not tmpdir: tmpdir = tempfile.mkdtemp()
+
+    ret_dirs = []
+    for d in dirs:
+        ret_dirs.append(create_dir(d, tmpdir=tmpdir))
+
+    ret_files = []
+    for f in files:
+        ret_files.append(create_file(f, tmpdir=tmpdir))
+
+    return tmpdir, ret_dirs, ret_files
 
-__version__ = '0.4'
 
 def create_dir(path, tmpdir=u""):
     '''
@@ -70,6 +144,25 @@ def create_file(path, contents=u"", tmpdir=u""):
 
     return file_path
 
+
+def create_modules(module_dict, tmpdir=u"", make_importable=True):
+    """
+    create a whole bunch of modules all at once
+
+    module_dict -- dict -- keys are the module_name, values are the module contents
+    tmpdir -- string -- same as create_module() tmpdir
+    make_importable -- boolean -- same as create_module() tmpdir
+    """
+    ret_modules = []
+    module_base_dir = create_dir(u"", tmpdir)
+
+    for module_name, contents in module_dict.iteritems():
+        ret_modules.append(create_module(module_name, contents, module_base_dir, make_importable))
+        make_importable = False
+
+    return ret_modules
+
+
 def create_module(module_name, contents=u"", tmpdir=u"", make_importable=True):
     '''
     create a python module folder structure so that the module can be imported
@@ -81,6 +174,21 @@ def create_module(module_name, contents=u"", tmpdir=u"", make_importable=True):
 
     return -- string -- the module file path
     '''
+    module_file = ''
+    mod_bits = filter(None, module_name.split(u'.'))
+    module_base_dir = create_dir(u"", tmpdir)
+    base_dir = module_base_dir
+    for modname in mod_bits:
+        base_dir = create_dir(modname, base_dir)
+        module_file = create_file(u"__init__.py", contents=contents, tmpdir=base_dir)
+
+    # add the path to the top of the sys path so importing the new module will work
+    if make_importable:
+        sys.path.insert(0, module_base_dir) 
+
+    return module_file
+
+
     mod_bits = filter(None, module_name.split(u'.'))
     module_base_dir = create_dir(u"", tmpdir)
     base_modname = mod_bits.pop()
@@ -189,7 +297,7 @@ def get_str(str_size=0, chars=None):
 
 def get_hex(str_size=0):
     '''
-    generate a string of just ascii characters
+    generate a string of just hex characters
 
     str_size -- integer -- how long you want the string to be
     return -- unicode
@@ -231,32 +339,62 @@ def get_float(min_size=None, max_size=None):
     
     return i
 
+def get_posint(max_size=2**31-1):
+    """
+    just return a positive 32-bit integer, this is basically a wrapper around
+    random.randint where you don't have to specify a minimum (or a maximum if you
+    don't want)
+    """
+    min_size = 1
+    return random.randint(min_size, max_size)
+
 def get_int(min_size=1, max_size=sys.maxsize):
+    return get_unique_int(min_size, max_size)
+
+def get_int32(min_size=1):
+    """returns a unique 32-bit positive integer"""
+    return get_unique_int(min_size, 2**31-1)
+
+def get_int64(min_size=1):
+    """returns up to a unique 64-bit positive integer"""
+    return get_unique_int(min_size, 2**63-1)
+
+def get_unique_int(min_size=1, max_size=sys.maxsize):
     '''
     get a random integer
 
     no different than random.randint except that it guarrantees no int will be
     the same, and also you don't have to set a range, it will default to all max
     int size
-    
+
     return -- integer 
     '''
     i = 0;
-    
-    while True:
+    found = False
+    max_count = max_size - min_size
+    for x in xrange(max_count):
         i = random.randint(min_size, max_size)
         if i not in _previous_ints:
+            found = True
             _previous_ints.add(i)
             break
-    
+
+    assert found, "no unique ints from {} to {} could be found".format(min_size, max_size)
     return i
 
-def get_words(word_count=0, as_str=True):
+def get_ascii_words(word_count=0, as_str=True):
+    return get_words(word_count, as_str, words=_ascii_words)
+
+def get_unicode_words(word_count=0, as_str=True):
+    return get_words(word_count, as_str, words=_unicode_words)
+
+def get_words(word_count=0, as_str=True, words=None):
     '''
     get some amount of random words
 
     word_count -- integer -- how many words you want, 0 means a random amount (at most 20)
     as_str -- boolean -- True to return as string, false to return as list of words
+    words -- list -- a list of words to choose from, defaults to unicode + ascii words
 
     return -- unicode|list -- your requested words
     '''
@@ -265,8 +403,34 @@ def get_words(word_count=0, as_str=True):
     if word_count == 0:
         word_count = random.randint(1, 20)
 
-    words = random.sample(_words, word_count)
-    return words if not as_str else u' '.join(words)
+    if not words:
+        words = _words
+
+    ret_words = random.sample(words, word_count)
+    return ret_words if not as_str else u' '.join(ret_words)
+
+def get_birthday(as_str=False):
+    """
+    return a random YYYY-MM-DD
+
+    as_str -- boolean -- true to return the bday as a YYYY-MM-DD string
+    return -- datetime.date|string
+    """
+    year = random.randint(1950, 1999)
+    month = random.randint(1, 12)
+    day = 1
+    if month == 2:
+        day = random.randint(1, 28)
+    elif month in [9, 4, 6, 11]:
+        day = random.randint(1, 30)
+    else:
+        day = random.randint(1, 31)
+
+    bday = datetime.date(year, month, day)
+    if as_str:
+        bday = "{:%Y-%m-%d}".format(bday)
+
+    return bday
 
 def get_email(name=u''):
     '''return a random email address'''
@@ -300,7 +464,7 @@ def get_email(name=u''):
         u"mailinator.com"
     ]
 
-    return u'{}@{}'.format(name, random.choice(email_domains))
+    return u'{}@{}'.format(name.lower(), random.choice(email_domains))
 
 def get_name(name_count=2, as_str=True):
     '''
@@ -338,13 +502,18 @@ def get_ascii_name():
     return random.choice(_names)
 
 def get_unicode_name():
-    '''
-    return one none ascii safe name
+    '''return one none ascii safe name'''
+    name = u''
+    while True:
+        # total hack to get around not all the names in _unicode_names being unicode
+        try:
+            name = random.choice(_unicode_names)
+            name.decode('utf-8')
+        except UnicodeEncodeError:
+            break
 
-    NOTE -- not all the names in _unicode_names are unicode, there are some ascii names in there
-    but I'm too lazy right now to go remove them
-    '''
-    return random.choice(_unicode_names)
+    return name
+    # return random.choice(_unicode_names)
 
 def get_coordinate(v1, v2, round_to=7):
     '''
@@ -821,10 +990,11 @@ Yuliya \u042f\u0440\u043e\u0441\u043b\u0430\u0432\u0430 Yaroslava \u042f\u043d\u
 
 Rene\u2019e A\u2019Laysyn, D\u2019Kota \u2019Ese Cam\u2019Ron Da\u2019neyelle No\u2019elle ZI\u2019eyekel Miche\u2019le
 """.strip())
+
 # via: http://www.lipsum.com/feed/html
 # russian is from: http://masterrussian.com/vocabulary/most_common_words.htm
 # japanese (4bytes) are from: http://www.i18nguy.com/unicode/supplementary-test.html
-_paragraphs = u'''
+_ascii_paragraphs = u'''
 Lorem ipsum dolor sit amet, consectetuer adipiscing elit. Phasellus
 pharetra urna sit amet magna. Donec posuere porta velit. Vestibulum sed libero.
 Ut vestibulum sodales arcu. Proin vulputate, mi quis luctus ornare, elit ligula fringilla nisi,
@@ -872,7 +1042,9 @@ Nunc laoreet. Morbi pharetra. Integer cursus molestie turpis. Nam cursus sodales
 Maecenas non lacus. Pellentesque habitant morbi tristique senectus et netus et malesuada fames
 ac turpis egestas. Nam vel nibh eu nulla blandit facilisis. Sed varius turpis ac neque.
 Curabitur vel erat. Morbi sed purus id erat tincidunt ullamcorper.
+'''
 
+_unicode_paragraphs = u'''
 \u0437\u043d\u0430\u0442\u044c \u043c\u043e\u0439 \u0434\u043e \u0438\u043b\u0438 \u0435\u0441\u043b\u0438
 \u0432\u0440\u0435\u043c\u044f \u0440\u0443\u043a\u0430 \u043d\u0435\u0442 \u0441\u0430\u043c\u044b\u0439
 \u043d\u0438 \u0441\u0442\u0430\u0442\u044c \u0431\u043e\u043b\u044c\u0448\u043e\u0439 \u0434\u0430\u0436\u0435
@@ -921,7 +1093,7 @@ Curabitur vel erat. Morbi sed purus id erat tincidunt ullamcorper.
 
 # only add 4-byte unicode if 4-byte unicode is supported
 if sys.maxunicode > 65535:
-    _paragraphs += u'''
+    _unicode_paragraphs += u'''
 \U0002070e \U00020731 \U00020779 \U00020c53 \U00020c78 \U00020c96 \U00020ccf \U00020cd5 \U00020d15 \U00020d7c
 \U00020d7f \U00020e0e \U00020e0f \U00020e77 \U00020e9d \U00020ea2 \U00020ed7 \U00020ef9 \U00020efa \U00020f2d
 \U00020f2e \U00020f4c \U00020fb4 \U00020fbc \U00020fea \U0002105c \U0002106f \U00021075 \U00021076 \U0002107b
@@ -931,7 +1103,9 @@ if sys.maxunicode > 65535:
 \U00028cd2 \U00029d98
 '''
 
-_words = re.split(r'\s+', _paragraphs.strip())
+_ascii_words = re.split(ur'\s+', _ascii_paragraphs.strip())
+_unicode_words = re.split(ur'\s+', _unicode_paragraphs.strip())
+_words = _ascii_words + _unicode_words
 
 def _normpath(path):
     '''
